/*
    This file is part of COMMON AVR UTILS.

    COMMON AVR UTILS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    COMMON AVR UTILS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include <avr/io.h> // RAMEND
#include "mem-check.h"

// Mask to init SRAM and check against
#define MASK 0xaa

// __heap_start is defined in the linker script
extern unsigned char __heap_start;

unsigned short 
get_mem_unused (void)
{
	unsigned short unused = 0;
    
    // Get end of static allocated RAM space (.data, .bss, .noinit, ...)
	unsigned char *p = &__heap_start;
	
	do
	{
        // Mask written in __init_mem still intact?
		if (*p++ != MASK)
			break;
			
		unused++;
	} while (p <= (unsigned char*) RAMEND);

	return unused;
}

// Init RAM space after static allocated RAM with MASK.
// Accesses to the stack will overwrite this mask, so
// we can probe later on and detect memory usage.
// !!! Do never call this function         !!!
// !!! The Linker knows what to do with it !!!
void __attribute__ ((naked, section (".init8")))
__init_mem (void) 
{
	__asm__ __volatile (
		"ldi r30, lo8 (%[start])"      "\n\t"
		"ldi r31, hi8 (%[start])"      "\n\t"
		"ldi r24, lo8 (%[mask])"       "\n\t"
		"ldi r25, hi8 (%[end])"        "\n\t"
		"0:"                           "\n\t"
		"st  Z+,  r24"                 "\n\t"
		"cpi r30, lo8 (%[end])"        "\n\t"
		"cpc r31, r25"                 "\n\t"
		"brlo 0b"
		: 
		: [mask]  "i" (MASK), 
          [end]   "i" (RAMEND+1), 
          [start] "i" (&__heap_start)
	);
}
