/*
    This file is part of COMMON AVR UTILS.

    COMMON AVR UTILS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    COMMON AVR UTILS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "uart.h"

#if defined (__AVR_ATmega88__) || defined (__AVR_ATmega168__) || defined (__AVR_ATmega328P__)
#define UCSRA UCSR0A
#define UCSRB UCSR0B
#define UCSRC UCSR0C
#define UDR   UDR0
#define UDRIE UDRIE0
#define RXEN  RXEN0
#define TXEN  TXEN0
#define RXC   RXC0
#define TXC   TXC0
#define TXCIE TXCIE0
#define RXCIE RXCIE0
#define UCSZ1 UCSZ01
#define UCSZ0 UCSZ00
#define UBRRL UBRR0L
#define UBRRH UBRR0H
#define U2X   U2X0
#define UDRE  UDRE0
#define SIG_UART_DATA  SIG_USART_DATA
#define SIG_UART_TRANS SIG_USART_TRANS
#define SIG_UART_RECV  SIG_USART_RECV
#endif

#if !defined (U2X_BIT)
#define U2X_BIT 1
#endif // U2X_BIT

#if !defined (BAUDRATE)
#error Define BAUDRATE
#endif // BAUDRATE

#if !defined (F_CPU)
#error Define F_CPU
#endif // F_CPU

void uart_init (void)
{
    unsigned short ubrr = -.6 + F_CPU/((16L-8*U2X_BIT)*BAUDRATE);

    UBRRH = ubrr >> 8;
    UBRRL = ubrr;

    UCSRA = (U2X_BIT << U2X) | (1 << TXC);

    // Enable UART Receiver, Transmitter
    // Data mode 8N1, asynchronous

    UCSRB = (1 << RXEN) | (1 << TXEN);

#if defined (URSEL)
    UCSRC = (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);
#else
    UCSRC = (1 << UCSZ1) | (1 << UCSZ0);
#endif
    do
        UDR;
    while (UCSRA & (1 << RXC));
}

char uart_getc(void)
{
    while (!(UCSRA & (1 << RXC)))
        ;
    return (char) UDR;
}

int uart_getc_nowait(void)
{
    return (UCSRA & (1 << RXC)) ? 0xff & UDR : -1;
}

/////////////////////////////////////////////////////////////////

void uart_putc (const char c)
{
    while (!(UCSRA & (1 << UDRE)))
        ;
    UDR = c;
}
