#include <avr/io.h>

#include "uput.h"
#include "uart.h"
#include "soft-reset.h"
#include "data-types.h"

// put_P ein vertrautes Aussehen geben
#define printf(FMT,X...)      put_P (FMT , ##X)
#define printf_P(FMT,X...)    put_PSTR (FMT , ##X)
#define sprintf(STR,FMT,X...) put_P ("%@" FMT, STR , ##X)

void init (void)
{
    uart_init ();
}

// Zahl in unterschiedlichen Geschmacksrichtungen ausgeben
void print_zahl (long z, uint8_t sig)
{
    if (sig)
        z = -z;
    short s = z;
    
    data32_t d = {.asLong = z};
    
    printf ("%%d: %d\n", s); // signed short
    printf ("%%u: %u\n", s); // unsigned short
    printf ("%%l: %l\n", z); // signed long
    printf ("%%x: 0x%x\n", s); // 0x00
    printf ("%%X: 0x%X\n", s); // 0x0000
    printf ("%%Y: 0x%Y\n", z); // 0x00000000
    printf ("%%q: %q\n", s); // Q 1.15
    printf ("%%Q: %Q\n", z); // Q 1.31
    printf ("%%v: %v\n", s); // Q 4.12
    printf ("%%b: 0b%b %b %b %b\n", d.asByte[3], d.asByte[2], d.asByte[1], d.asByte[0]); // 0b00000000
    
    printf ("\n");
}

char text[30];

#define ESC '\e'

// String wird 2x verwendet. Zum Platzsparen eine Variable verwenden.
const char MEM_STR[] PROGMEM 
    = "Freier RAM-Speicher: %M Bytes%s";

int main (void)
{
    init();
    
    // Callback: muss initialisiert werden!
    // Entweder hier oder per -DPUT_CHAR_INIT=...
    put_char = uart_putc;
    
    // Um den Speicherverbrauch bestimmen zu knnen
    extern unsigned char __heap_start;
    extern unsigned char __data_start;
    
    int heap_start = (int) & __heap_start;
    int data_start = (int) & __data_start;
    
    printf ("\n\nHallo, hier ist die Demo.\n");
    printf_P (MEM_STR, " ");
    printf ("von %d Bytes RAM insgesamt,\n"
            "davon statisch belegt: %d Bytes\n", 
                RAMEND+1 - data_start, heap_start - data_start);
    
    printf ("ESC -> Reset\n");
    printf ("M   -> RAM-Verbrauch\n");
    printf ("- 1 2 3 4 5 6 7 8 9 0 -> Ziffern eingeben\n");
    printf ("ENTER                 -> Zahl auf 0 setzen\n");
    
    long zahl = 0;
    uint8_t zero = 1;
    uint8_t sign = 0;
    
    while (1)
    {
        // Ein Zeichen von serieller Schnittstelle lesen
        // 9600 Baud, 8N1, keine Flusteuerung
        int c = uart_getc_nowait ();
        
        // Reset
        if (c == ESC)
            soft_reset();
            
        // RAM-Verbrach    
        if (c == 'M')
            printf_P (MEM_STR, "\n\r");
        
        // Zahl eingeben
        if (c >= '0' && c <= '9')
        {
            zero = 0;
            zahl = zahl*10 + (c - '0');
            print_zahl (zahl, sign);
            continue;
        }
            
        if (c == '-' && zero)
        {
            sign = 1;
            zero = 0;
            printf ("-");
            continue;
        }
        
        if (!zero && c == '\n')
        {
            printf ("######\n");
            zahl = 0;
            zero = 1;
            sign = 0;
            continue;
        }
        
        if (c >= 0 && c != '\r' && c != '\n')
        {
            zahl = 0; sign = 0; zero = 1;
            sprintf (text, "{0x%x='%c'}", c, c);
            printf  ("%s\n", text);
        }
    }

    return 0;
}
