/*
    This file is part of COMMON AVR UTILS.

    COMMON AVR UTILS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    COMMON AVR UTILS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef _UPUT_H_
#define _UPUT_H_

#include <inttypes.h>
#include <avr/pgmspace.h>

// mind the blank before the colon!
#define put_P(y,z...)   put_PSTR (PSTR(y) , ##z)
extern void put_PSTR (const char *, ...);
extern void put_str  (const char*);
extern void put_pstr (const char*);

extern void (*put_char)(char);
/*
First  column: activation bit in mask
Second column: activation mask

To activate %M, %u and %d write
	#define PUTTER 0x1006
or call 
	gcc -DPUTTER=0x1006 ...

Strings, Chars are always active
                    %s :  String (RAM)
                    %S :  String (Flash)
                    %c :  one char
                    %% :  %

Decimal
     0   0x0001     %2 :  unsigned  8-bit dec-val, 2 digits
     1   0x0002     %u :  unsigned 16-bit int, no leading zeros
     2   0x0004     %d :  signed   16-bit int, no leading zeros
    13   0x2000     %l :  signed   32-bit int, no leading zeros
                  
Hex
     3   0x0008     %x :  unsigned  8-bit hex-val, 2 digits, leading zeros
     4   0x0010     %X :  unsigned 16-bit hex-val, 4 digits, leading zeros
     5   0x0020     %Y :  unsigned 32-bit hex-val, 8 digits, leading zeros
                  
Binary
     6   0x0040     %b :  unsigned  8-bit bin-val, 8 digits

Q-Format
     7   0x0080     %q :  signed 1.15-bit fraction
     8   0x0100     %Q :  signed 1.31-bit fraction
     9   0x0200     %v :  signed 4.12-bit fraction
                  
Misc
    10   0x0400     %F :  set output function void (*)(char)
    11   0x0800     %@ :  write to respective address
                          resulting string will be 0-terminated
    12   0x1000     %M :  print memory usage, link against mem-check!
*/

#endif // _UPUT_H_
